package cn.com.qmth.examcloud.web.filestorage.impl;

import java.io.File;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.web.filestorage.FileStorage;
import cn.com.qmth.examcloud.web.filestorage.FileStoragePathEnvInfo;
import cn.com.qmth.examcloud.web.filestorage.FileStorageType;
import cn.com.qmth.examcloud.web.filestorage.FileStorageUtil;
import cn.com.qmth.examcloud.web.filestorage.YunHttpRequest;
import cn.com.qmth.examcloud.web.filestorage.YunPathInfo;
import cn.com.qmth.examcloud.web.upyun.UpYunClient;
import cn.com.qmth.examcloud.web.upyun.UpYunHttpRequest;
import cn.com.qmth.examcloud.web.upyun.UpYunPathInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunPathEnvironmentInfo;
import cn.com.qmth.examcloud.web.upyun.UpyunService;
import cn.com.qmth.examcloud.web.upyun.UpyunSite;
import cn.com.qmth.examcloud.web.upyun.UpyunSiteManager;

@Service(value = "upyunFileStorage")
public class UpyunFileStorageImpl implements FileStorage {
	
	@Autowired
	private UpyunService upyunService;

	@Override
	public YunPathInfo saveFile(File file, String path) {
		String siteId="transPath";
		UpyunPathEnvironmentInfo env = new UpyunPathEnvironmentInfo();
		env.setRelativePath(path);
		String fpath = upyunService.writeFile(siteId, env, file, null).getRelativePath();
		UpyunSite site=UpyunSiteManager.getUpyunSite(siteId);
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(site.getUpyunId());
		String relativePath=FileStorageType.UPYUN+"-"+site.getUpyunId()+"://"+fpath;
		String url=FileStorageUtil.getUrl(c.getDomain(), fpath);
		YunPathInfo pi=new YunPathInfo(url, relativePath);
		return pi;
	}

	@Override
	public String realPath(String path) {
		String upyunId=FileStorageUtil.getYunId(path);
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(upyunId);
		return FileStorageUtil.getUrl(c.getDomain(), path);
	}

	@Override
	public YunPathInfo saveFile(String siteId, FileStoragePathEnvInfo env, File file,String md5) {
		UpYunPathInfo pathInfo =upyunService.writeFile(siteId, of(env), file, md5);
		UpyunSite site=UpyunSiteManager.getUpyunSite(siteId);
		String relativePath=FileStorageType.UPYUN+"-"+site.getUpyunId()+"://"+pathInfo.getRelativePath();
		UpYunClient c=UpyunSiteManager.getUpYunClientByUpyunId(site.getUpyunId());
		String url=FileStorageUtil.getUrl(c.getDomain(), pathInfo.getRelativePath());
		YunPathInfo pi=new YunPathInfo(url, relativePath);
		return pi;
	}
	
	private UpyunPathEnvironmentInfo of(FileStoragePathEnvInfo env) {
		UpyunPathEnvironmentInfo ret=new UpyunPathEnvironmentInfo();
		ret.setExt1(env.getExt1());
		ret.setExt2(env.getExt2());
		ret.setExt3(env.getExt3());
		ret.setExt4(env.getExt4());
		ret.setExt5(env.getExt5());
		ret.setFileSuffix(env.getFileSuffix());
		ret.setRelativePath(env.getRelativePath());
		ret.setRootOrgDomain(env.getRootOrgDomain());
		ret.setRootOrgId(env.getRootOrgId());
		ret.setTimeMillis(env.getTimeMillis());
		ret.setUserId(env.getUserId());
		return ret;
	}

	@Override
	public YunHttpRequest getSignature(String siteId, FileStoragePathEnvInfo env, String md5) {
		UpYunHttpRequest req=upyunService.buildUpYunHttpRequest(siteId, of(env), md5);
		YunHttpRequest ret=new YunHttpRequest();
		ret.setAccessUrl(req.getAccessUrl());
		ret.setFormParams(req.getFormParams());
		ret.setFormUrl(req.getFormUrl());
		return ret;
	}

	@Override
	public void deleteFile(String path) {
		// TODO Auto-generated method stub
		
	}

}
